@@ -20,7 +20,7 @@ mod cli {
     use clap::Parser;
     use grex::RegExpBuilder;
     use itertools::Itertools;
-    use std::io::{BufRead, Error, ErrorKind, Read};
+    use std::io::{stdin, BufRead, Error, ErrorKind, IsTerminal, Read};
     use std::path::PathBuf;
 
     #[derive(Parser)]
@@ -42,6 +42,8 @@ mod cli {
         // --------------------
         /// One or more test cases separated by blank space
         ///
+        /// Use a hyphen `-` to read test cases from standard input.
+        ///
         /// Conflicts with --file.
         #[arg(
             value_name = "INPUT",
@@ -58,6 +60,8 @@ mod cli {
         /// Lines may be ended with either a newline `\n` or a carriage return with a line feed `\r\n`.
         /// The final line ending is optional.
         ///
+        /// Use a hyphen `-` to read the filename from standard input.
+        ///
         /// Conflicts with INPUT...
         #[arg(
             name = "file",
@@ -287,14 +291,14 @@ mod cli {
     }
 
     pub(crate) fn obtain_input(cli: &Cli) -> Result<Vec<String>, Error> {
-        let is_stdin_available = atty::isnt(atty::Stream::Stdin);
+        let is_stdin_available = !stdin().is_terminal();
 
         if !cli.input.is_empty() {
             let is_single_item = cli.input.len() == 1;
             let is_hyphen = cli.input.get(0).unwrap() == "-";
 
             if is_single_item && is_hyphen && is_stdin_available {
-                Ok(std::io::stdin()
+                Ok(stdin()
                     .lock()
                     .lines()
                     .map(|line| line.unwrap())
@@ -306,12 +310,12 @@ mod cli {
             let is_hyphen = file_path.as_os_str() == "-";
             let path = if is_hyphen && is_stdin_available {
                 let mut stdin_file_path = String::new();
-                std::io::stdin().read_to_string(&mut stdin_file_path)?;
+                stdin().read_to_string(&mut stdin_file_path)?;
                 PathBuf::from(stdin_file_path.trim())
             } else {
                 file_path.to_path_buf()
             };
-            match std::fs::read_to_string(&path) {
+            match std::fs::read_to_string(path) {
                 Ok(file_content) => Ok(file_content.lines().map(|it| it.to_string()).collect_vec()),
                 Err(error) => Err(error),
             }
@@ -323,7 +327,10 @@ mod cli {
         }
     }
 
-    pub(crate) fn handle_input(cli: &Cli, input: Result<Vec<String>, Error>) {
+    pub(crate) fn handle_input(
+        cli: &Cli,
+        input: Result<Vec<String>, Error>,
+    ) -> Result<(), Box<dyn std::error::Error>> {
         match input {
             Ok(test_cases) => {
                 let mut builder = RegExpBuilder::from(&test_cases);
@@ -397,16 +404,17 @@ mod cli {
                 let regexp = builder.build();
 
                 println!("{}", regexp);
+                Ok(())
             }
             Err(error) => match error.kind() {
-                ErrorKind::NotFound => eprintln!("error: the specified file could not be found"),
+                ErrorKind::NotFound => Err("error: the specified file could not be found".into()),
                 ErrorKind::InvalidData => {
-                    eprintln!("error: the specified file's encoding is not valid UTF-8")
+                    Err("error: the specified file's encoding is not valid UTF-8".into())
                 }
                 ErrorKind::PermissionDenied => {
-                    eprintln!("permission denied: the specified file could not be opened")
+                    Err("permission denied: the specified file could not be opened".into())
                 }
-                _ => eprintln!("error: {}", error),
+                _ => Err(format!("error: {}", error).into()),
             },
         }
     }
@@ -429,7 +437,10 @@ mod cli {
 fn main() {
     use clap::Parser;
     let cli = cli::Cli::parse();
-    cli::handle_input(&cli, cli::obtain_input(&cli));
+    if let Err(e) = cli::handle_input(&cli, cli::obtain_input(&cli)) {
+        eprintln!("{}", e);
+        std::process::exit(1);
+    }
 }
 
 #[cfg(target_family = "wasm")]
